@@ -1,4 +1,4 @@
-from os.path import abspath
+from os.path import abspath, join, dirname
 from relatorio import Report
 
 # test data
@@ -7,8 +7,9 @@ from common import inv
 #PDF
 if __name__ == '__main__':
     print "generating output_basic.pdf... ",
-    report = Report(abspath('basic.tex'), 'application/pdf')
+    report = Report(abspath(join(dirname(__file__), 'basic.tex')),
+        'application/pdf')
     content = report(o=inv).render().getvalue()
-    file('output_basic.pdf', 'wb').write(content)
+    file(join(dirname(__file__), 'output_basic.pdf'), 'wb').write(content)
     print "done"
 
