@@ -1,11 +1,12 @@
 ###############################################################################
 #
+# Copyright (c) 2009 Cedric Krier.
 # Copyright (c) 2007, 2008 OpenHex SPRL. (http://openhex.com) All Rights
 # Reserved.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
+# Foundation; either version 3 of the License, or (at your option) any later
 # version.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT
@@ -44,6 +45,7 @@ from genshi.template import MarkupTemplate
 from genshi.filters import Transformer
 from genshi.filters.transform import ENTER, EXIT
 from genshi.core import Stream
+from genshi.template.interpolation import PREFIX
 
 
 from relatorio.templates.base import RelatorioStream
@@ -72,6 +74,7 @@ EXTENSIONS = {'image/png': 'png',
 
 RELATORIO_URI = 'http://relatorio.openhex.org/'
 GENSHI_URI = 'http://genshi.edgewall.org/'
+MANIFEST = 'META-INF/manifest.xml'
 output_encode = genshi.output.encode
 EtreeElement = lxml.etree.Element
 
@@ -97,7 +100,7 @@ EtreeElement = lxml.etree.Element
 def guess_type(val):
     if isinstance(val, (str, unicode)):
         return 'string'
-    elif isinstance(val, (int, float)):
+    elif isinstance(val, (int, float, long)):
         return 'float'
 
 class OOTemplateError(genshi.template.base.TemplateSyntaxError):
@@ -107,12 +110,13 @@ class OOTemplateError(genshi.template.base.TemplateSyntaxError):
 class ImageHref:
     "A class used to add images in the odf zipfile"
 
-    def __init__(self, zfile, context):
+    def __init__(self, zfile, manifest, context):
         self.zip = zfile
+        self.manifest = manifest
         self.context = context.copy()
 
     def __call__(self, expr):
-        bitstream, mimetype = expr
+        bitstream, mimetype = expr[:2]
         if isinstance(bitstream, Report):
             bitstream = bitstream(**self.context).render()
         elif isinstance(bitstream, ChartTemplate):
@@ -123,9 +127,29 @@ class ImageHref:
         path = 'Pictures/%s.%s' % (name, EXTENSIONS[mimetype])
         if path not in self.zip.namelist():
             self.zip.writestr(path, file_content)
+            self.manifest.add_file_entry(path, mimetype)
         return {'{http://www.w3.org/1999/xlink}href': path}
 
 
+class ImageDimension:
+    "A class used to set dimension in draw tags"
+
+    def __init__(self, namespaces):
+        self.namespaces = namespaces
+
+    def __call__(self, expr, width, height):
+        # expr could be (bitstream, mimetype)
+        # or (bitstreamm mimetype, width, height)
+        if len(expr) == 4:
+            width, height = expr[2:]
+        attrs = {}
+        if width:
+            attrs['{%s}width' % self.namespaces['svg']] = width
+        if height:
+            attrs['{%s}height' % self.namespaces['svg']] = height
+        return attrs
+
+
 class ColumnCounter:
     """A class used to count the actual maximum number of cells (and thus
     columns) a table contains accross its rows.
@@ -168,6 +192,20 @@ def wrap_nodes_between(first, last, new_parent):
     old_parent.remove(last)
 
 
+def update_py_attrs(node, value):
+    """An helper function to update py_attrs of a node.
+    """
+    if not value:
+        return
+    py_attrs_attr = '{%s}attrs' % GENSHI_URI
+    if not py_attrs_attr in node.attrib:
+        node.attrib[py_attrs_attr] = value
+    else:
+        node.attrib[py_attrs_attr] = \
+                "(lambda x, y: x.update(y) or x)(%s or {}, %s or {})" % \
+                (node.attrib[py_attrs_attr], value)
+
+
 class Template(MarkupTemplate):
 
     def __init__(self, source, filepath=None, filename=None, loader=None,
@@ -227,7 +265,8 @@ class Template(MarkupTemplate):
             "draw": "urn:draw",
             "table": "urn:table",
             "office": "urn:office",
-            "xlink": "urn:xlink"
+            "xlink": "urn:xlink",
+            "svg": "urn:svg",
         }
         # but override them with the real namespaces
         self.namespaces.update(root.nsmap)
@@ -242,6 +281,7 @@ class Template(MarkupTemplate):
         self._handle_relatorio_tags(tree)
         self._handle_images(tree)
         self._handle_innerdocs(tree)
+        self._escape_values(tree)
         return StringIO(lxml.etree.tostring(tree))
 
     def _invert_style(self, tree):
@@ -397,8 +437,8 @@ class Template(MarkupTemplate):
                 # The grand-parent tag is a table cell we should set the
                 # correct value and type for this cell.
                 dico = "{'%s': %s, '%s': __relatorio_guess_type(%s)}"
-                parent.attrib[py_attrs_attr] = dico % (office_name, expr,
-                                                       office_valuetype, expr)
+                update_py_attrs(parent, dico %
+                        (office_name, expr, office_valuetype, expr))
                 parent.attrib.pop(office_valuetype, None)
                 parent.attrib.pop(office_name, None)
 
@@ -428,16 +468,15 @@ class Template(MarkupTemplate):
         # Note that table_name is not needed in the first two
         # operations, but a unique id within the table is required
         # to support nested column repetition
-        ancestor.attrib[py_attrs_attr] = \
-            "__relatorio_reset_col_count(%d)" % loop_id
+        update_py_attrs(ancestor, "__relatorio_reset_col_count(%d)" % loop_id)
 
         # 2) add increment code (through a py:attrs attribute) on
         #    the first cell node after the opening (cell node)
         #    ancestor
         enclosed_cell = outer_o_node.getnext()
         assert enclosed_cell.tag == '{%s}table-cell' % table_namespace
-        enclosed_cell.attrib[py_attrs_attr] = \
-            "__relatorio_inc_col_count(%d)" % loop_id
+        update_py_attrs(enclosed_cell, "__relatorio_inc_col_count(%d)" %
+                loop_id)
 
         # 3) add "store count" code as a py:replace node, as the
         #    last child of the row
@@ -587,15 +626,23 @@ class Template(MarkupTemplate):
         draw_name = '{%s}name' % draw_namespace
         draw_image = '{%s}image' % draw_namespace
         py_attrs = '{%s}attrs' % self.namespaces['py']
+        svg_namespace = self.namespaces['svg']
+        svg_width = '{%s}width' % svg_namespace
+        svg_height = '{%s}height' % svg_namespace
         xpath_expr = "//draw:frame[starts-with(@draw:name, 'image:')]"
         for draw in tree.xpath(xpath_expr, namespaces=self.namespaces):
-            d_name = draw.attrib[draw_name]
-            attr_expr = "__relatorio_make_href(%s)" % d_name[7:]
+            d_name = draw.attrib[draw_name][6:].strip()
+            attr_expr = "__relatorio_make_href(%s)" % d_name
             image_node = EtreeElement(draw_image,
                                       attrib={py_attrs: attr_expr},
                                       nsmap={'draw': draw_namespace,
                                              'py': GENSHI_URI})
             draw.replace(draw[0], image_node)
+            width = draw.attrib.pop(svg_width, None)
+            height = draw.attrib.pop(svg_height, None)
+            attr_expr = "__relatorio_make_dimension(%s, '%s', '%s')" % \
+                    (d_name, width, height)
+            draw.attrib[py_attrs] = attr_expr
 
     def _handle_innerdocs(self, tree):
         "finds inner_docs and adds them to the processing stack."
@@ -605,10 +652,23 @@ class Template(MarkupTemplate):
         for draw in tree.xpath(xpath_expr, namespaces=self.namespaces):
             self.inner_docs.append(draw.attrib[href_attrib][2:])
 
+    def _escape_values(self, tree):
+        "escapes element values"
+        for element in tree.iter():
+            for attrs in element.keys():
+                if not attrs.startswith('{%s}' % GENSHI_URI):
+                    element.attrib[attrs] = element.attrib[attrs]\
+                            .replace(PREFIX, PREFIX * 2)
+            if element.text:
+                element.text = element.text.replace(PREFIX, PREFIX * 2)
+
     def generate(self, *args, **kwargs):
         "creates the RelatorioStream."
         serializer = OOSerializer(self.filepath)
-        kwargs['__relatorio_make_href'] = ImageHref(serializer.outzip, kwargs)
+        kwargs['__relatorio_make_href'] = ImageHref(serializer.outzip,
+                                                    serializer.manifest,
+                                                    kwargs)
+        kwargs['__relatorio_make_dimension'] = ImageDimension(self.namespaces)
         kwargs['__relatorio_guess_type'] = guess_type
 
         counter = ColumnCounter()
@@ -657,10 +717,33 @@ class DuplicateColumnHeaders(object):
                 yield mark, (kind, data, pos)
 
 
+class Manifest(object):
+
+    def __init__(self, content):
+        self.tree = lxml.etree.parse(StringIO(content))
+        self.root = self.tree.getroot()
+        self.namespaces = self.root.nsmap
+
+    def __str__(self):
+        return lxml.etree.tostring(self.tree, encoding='UTF-8',
+                                   xml_declaration=True)
+
+    def add_file_entry(self, path, mimetype=None):
+        manifest_namespace = self.namespaces['manifest']
+        attribs = {'media-type': mimetype or '',
+                   'full-path': path}
+        entry_node = EtreeElement('{%s}%s' % (manifest_namespace,
+                                              'file-entry'),
+                                  attrib=attribs,
+                                  nsmap={'manifest': manifest_namespace})
+        self.root.append(entry_node)
+
+
 class OOSerializer:
 
     def __init__(self, oo_path):
         self.inzip = zipfile.ZipFile(oo_path)
+        self.manifest = Manifest(self.inzip.read(MANIFEST))
         self.new_oo = StringIO()
         self.outzip = zipfile.ZipFile(self.new_oo, 'w')
         self.xml_serializer = genshi.output.XMLSerializer()
@@ -686,6 +769,8 @@ class OOSerializer:
                     setattr(new_info, attr, getattr(f_info, attr))
                 serialized_stream = output_encode(self.xml_serializer(stream))
                 self.outzip.writestr(new_info, serialized_stream)
+            elif f_info.filename == MANIFEST:
+                self.outzip.writestr(f_info, str(self.manifest))
             else:
                 self.outzip.writestr(f_info, self.inzip.read(f_info.filename))
         self.inzip.close()
